Ext.namespace('Zarafa.owncloud.settings');

/**
 * @class Zarafa.owncloud.settings.SettingsOwncloudComponentsWidget
 * @extends Zarafa.settings.ui.SettingsWidget
 * @xtype zarafa.settingsowncloudwidget
 *
 * The {@link Zarafa.settings.ui.SettingsWidget widget} for configuring
 * the general owncloud options in the {@link Zarafa.owncloud.settings.SettingsOwncloudComponentsCategory owncloud category}.
 */
Zarafa.owncloud.settings.SettingsOwncloudComponentsWidget = Ext.extend(Zarafa.settings.ui.SettingsWidget, {

	/**
	 * @constructor
	 * @param {Object} config Configuration object
	 */
	constructor : function(config)
	{
		config = config || {};

		Ext.applyIf(config, {
			title : _('Enable/Disable owncloud components'),
			layout : 'form',
			items : [{
				xtype : 'checkbox',
				name : 'zarafa/v1/plugins/owncloud/enable_attach_from_oc',
				ref : 'attachFrom',
				fieldLabel : _('Attach a file from owncloud'),
				lazyInit : false,
				listeners : {
					check : this.onCheck,
					scope : this
				}
			},{
				xtype : 'checkbox',
				name : 'zarafa/v1/plugins/owncloud/enable_store_to_oc',
				ref : 'attachTo',
				fieldLabel : _('Save received attachments to owncloud'),
				lazyInit : false,
				listeners : {
					check : this.onCheck,
					scope : this
				}
			}]
		});

		Zarafa.owncloud.settings.SettingsOwncloudComponentsWidget.superclass.constructor.call(this, config);
	},

	/**
	 * Called by the {@link Zarafa.settings.ui.SettingsCategory Category} when
	 * it has been called with {@link zarafa.settings.ui.SettingsCategory#update}.
	 * This is used to load the latest version of the settings from the
	 * {@link Zarafa.settings.SettingsModel} into the UI of this category.
	 * @param {Zarafa.settings.SettingsModel} settingsModel The settings to load
	 */
	update : function(settingsModel)
	{
		this.model = settingsModel;
		this.attachFrom.setValue(settingsModel.get(this.attachFrom.name));
		this.attachTo.setValue(settingsModel.get(this.attachTo.name));
	},

	/**
	 * Called by the {@link Zarafa.settings.ui.SettingsCategory Category} when
	 * it has been called with {@link zarafa.settings.ui.SettingsCategory#updateSettings}.
	 * This is used to update the settings from the UI into the {@link Zarafa.settings.SettingsModel settings model}.
	 * @param {Zarafa.settings.SettingsModel} settingsModel The settings to update
	 */
	updateSettings : function(settingsModel)
	{
		settingsModel.set(this.attachFrom.name, this.attachFrom.getValue());
		settingsModel.set(this.attachTo.name, this.attachTo.getValue());
	},

	/**
	 * Event handler called when checkbox has been modified
	 *
	 * @param {Ext.form.CheckBox} checkbox Checkbox element from which the event originated
	 * @param {Boolean} checked State of the checkbox
	 * @private
	 */
	onCheck : function(checkbox, checked)
	{
		if(this.model) {
			// FIXME: The settings model should be able to detect if
			// a change was applied
			if (this.model.get(checkbox.name) !== checked) {
				this.model.set(checkbox.name, checked);
			}
		}
	}
});

Ext.reg('zarafa.settingsowncloudcomponentswidget', Zarafa.owncloud.settings.SettingsOwncloudComponentsWidget);
