Ext.namespace('Zarafa.owncloud.settings');

/**
 * @class Zarafa.owncloud.settings.SettingsOwncloudWidget
 * @extends Zarafa.settings.ui.SettingsWidget
 * @xtype zarafa.settingsowncloudwidget
 *
 * The {@link Zarafa.settings.ui.SettingsWidget widget} for configuring
 * the general owncloud options in the {@link Zarafa.owncloud.settings.SettingsOwncloudCategory owncloud category}.
 */
Zarafa.owncloud.settings.SettingsOwncloudWidget = Ext.extend(Zarafa.settings.ui.SettingsWidget, {

	Base64 : {
		// private property
		_keyStr : "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",

		// public method for encoding
		encode : function (input) {
			var output = "";
			var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
			var i = 0;

			input = this._utf8_encode(input);

			while (i < input.length) {

				chr1 = input.charCodeAt(i++);
				chr2 = input.charCodeAt(i++);
				chr3 = input.charCodeAt(i++);

				enc1 = chr1 >> 2;
				enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
				enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
				enc4 = chr3 & 63;

				if (isNaN(chr2)) {
					enc3 = enc4 = 64;
				} else if (isNaN(chr3)) {
					enc4 = 64;
				}

				output = output +
				this._keyStr.charAt(enc1) + this._keyStr.charAt(enc2) +
				this._keyStr.charAt(enc3) + this._keyStr.charAt(enc4);

			}

			return output;
		},
		
		// public method for decoding
		decode : function (input) {
			var output = "";
			var chr1, chr2, chr3;
			var enc1, enc2, enc3, enc4;
			var i = 0;

			input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");

			while (i < input.length) {

				enc1 = this._keyStr.indexOf(input.charAt(i++));
				enc2 = this._keyStr.indexOf(input.charAt(i++));
				enc3 = this._keyStr.indexOf(input.charAt(i++));
				enc4 = this._keyStr.indexOf(input.charAt(i++));

				chr1 = (enc1 << 2) | (enc2 >> 4);
				chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
				chr3 = ((enc3 & 3) << 6) | enc4;

				output = output + String.fromCharCode(chr1);

				if (enc3 != 64) {
					output = output + String.fromCharCode(chr2);
				}
				if (enc4 != 64) {
					output = output + String.fromCharCode(chr3);
				}

			}

			output = this._utf8_decode(output);

			return output;

		},
		
		// private method for UTF-8 decoding
		_utf8_decode : function (utftext) {
			var string = "";
			var i = 0;
			var c = c1 = c2 = 0;

			while ( i < utftext.length ) {

				c = utftext.charCodeAt(i);

				if (c < 128) {
					string += String.fromCharCode(c);
					i++;
				}
				else if((c > 191) && (c < 224)) {
					c2 = utftext.charCodeAt(i+1);
					string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
					i += 2;
				}
				else {
					c2 = utftext.charCodeAt(i+1);
					c3 = utftext.charCodeAt(i+2);
					string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
					i += 3;
				}

			}

			return string;
		},

		// private method for UTF-8 encoding
		_utf8_encode : function (string) {
			string = string.replace(/\r\n/g,"\n");
			var utftext = "";

			for (var n = 0; n < string.length; n++) {

				var c = string.charCodeAt(n);

				if (c < 128) {
					utftext += String.fromCharCode(c);
				}
				else if((c > 127) && (c < 2048)) {
					utftext += String.fromCharCode((c >> 6) | 192);
					utftext += String.fromCharCode((c & 63) | 128);
				}
				else {
					utftext += String.fromCharCode((c >> 12) | 224);
					utftext += String.fromCharCode(((c >> 6) & 63) | 128);
					utftext += String.fromCharCode((c & 63) | 128);
				}

			}

			return utftext;
		}
	},

	/**
	 * @constructor
	 * @param {Object} config Configuration object
	 */
	constructor : function(config)
	{
		config = config || {};

		Ext.applyIf(config, {
			title : _('General owncloud settings'),
			layout : 'form',
			items : [{
				xtype : 'textfield',
				name : 'zarafa/v1/contexts/owncloud/server',
				ref : 'serverField',
				fieldLabel : _('Server Address'),
				emptyText : _('demo.owncloud.org'),
				anchor : '100%',
				listeners : {
					change : this.onFieldChange,
					scope : this
				}
			},{
				xtype : 'textfield',
				name : 'zarafa/v1/contexts/owncloud/owncloud_path',
				ref : 'pathField',
				fieldLabel : _('Path to Owncloud/Webdav'),
				emptyText : _('/files/webdav.php'),
				anchor : '100%',
				listeners : {
					change : this.onFieldChange,
					scope : this
				}
			},{
				xtype : 'textfield',
				name : 'zarafa/v1/contexts/owncloud/username',
				ref : 'usernameField',
				fieldLabel : _('Username'),
				emptyText : _('test'),
				anchor : '100%',
				listeners : {
					change : this.onFieldChange,
					scope : this
				}
			},{
				xtype : 'textfield',
				name : 'zarafa/v1/contexts/owncloud/password',
				ref : 'passwordField',
				inputType : 'password',
				fieldLabel : _('Password'),
				emptyText : _('test'),
				anchor : '100%',
				listeners : {
					change : this.onFieldChange,
					scope : this
				}
			},{
				xtype : 'checkbox',
				name : 'zarafa/v1/contexts/owncloud/use_ssl',
				ref : 'useSSL',
				fieldLabel : _('Use SSL connections'),
				lazyInit : false,
				listeners : {
					check : this.onCheck,
					scope : this
				}
			}]
		});

		Zarafa.owncloud.settings.SettingsOwncloudWidget.superclass.constructor.call(this, config);
	},

	/**
	 * Called by the {@link Zarafa.settings.ui.SettingsCategory Category} when
	 * it has been called with {@link zarafa.settings.ui.SettingsCategory#update}.
	 * This is used to load the latest version of the settings from the
	 * {@link Zarafa.settings.SettingsModel} into the UI of this category.
	 * @param {Zarafa.settings.SettingsModel} settingsModel The settings to load
	 */
	update : function(settingsModel)
	{
		this.model = settingsModel;
		this.serverField.setValue(settingsModel.get(this.serverField.name));
		this.pathField.setValue(settingsModel.get(this.pathField.name));
		this.usernameField.setValue(settingsModel.get(this.usernameField.name));
		this.passwordField.setValue(this.Base64.decode(settingsModel.get(this.passwordField.name)));
		this.useSSL.setValue(settingsModel.get(this.useSSL.name));
	},

	/**
	 * Called by the {@link Zarafa.settings.ui.SettingsCategory Category} when
	 * it has been called with {@link zarafa.settings.ui.SettingsCategory#updateSettings}.
	 * This is used to update the settings from the UI into the {@link Zarafa.settings.SettingsModel settings model}.
	 * @param {Zarafa.settings.SettingsModel} settingsModel The settings to update
	 */
	updateSettings : function(settingsModel)
	{
		// We must either set the requested subject, or the default subject
		var server = this.serverField.getValue() || this.serverField.emptyText;
		var path = this.pathField.getValue() || this.pathField.emptyText;
		var username = this.usernameField.getValue() || this.usernameField.emptyText;
		var password = this.passwordField.getValue() || this.passwordField.emptyText;
		
		settingsModel.beginEdit();
		settingsModel.set(this.serverField.name, server);
		settingsModel.set(this.pathField.name, path);
		settingsModel.set(this.usernameField.name, username);
		settingsModel.set(this.passwordField.name, this.Base64.encode(password));
		settingsModel.set(this.useSSL.name, this.useSSL.getValue());
		settingsModel.endEdit();
		
		container.getNotifier().notify('info', 'Saved', 'Owncloud settings saved!');
	},

	/**
	 * Event handler called when checkbox has been modified
	 *
	 * @param {Ext.form.CheckBox} checkbox Checkbox element from which the event originated
	 * @param {Boolean} checked State of the checkbox
	 * @private
	 */
	onCheck : function(checkbox, checked)
	{
		if(this.model) {
			// FIXME: The settings model should be able to detect if
			// a change was applied
			if (this.model.get(checkbox.name) !== checked) {
				this.model.set(checkbox.name, checked);
			}
		}
	},
	
	/**
	 * Event handler which is called when one of the textfields has been changed.
	 * This will apply the new value to the settings (or if the text is empty,
	 * the {@link Ext.form.Field#emptyText} will be saved).
	 * @param {Ext.form.Field} field The field which has fired the event
	 * @param {String} value The new value
	 * @private
	 */
	onFieldChange : function(field, value)
	{
		if (this.model) {
			this.model.set(field.name, value || field.emptyText);
		}
	}
});

Ext.reg('zarafa.settingsowncloudwidget', Zarafa.owncloud.settings.SettingsOwncloudWidget);
